/*!
 * @file new_class_list.h
 * @brief Definition of a dynamically allocating ClassID
 *
 */

#ifndef _NEW_CLASS_LIST_H
#define _NEW_CLASS_LIST_H

#include "type_info.h"
#include <set>
#include <string>

#define DeclareClass(ClassName) \
   NewClassList ClassID_##ClassName(ClassName)

#define DeclareInClass(ClassName) \
   NewClassList classList(ClassName)

class NewClassListBase
{
public:
  int id() const { return _id; };
  const std::string& name() const { return _name; };

  static int classCount() { return _idCounter; };

protected:
  NewClassListBase(const std::string& className);
private:
  NewClassListBase(const NewClassListBase&);


private:
  int                           _id;
  std::string                   _name;

private:
  static int                           _idCounter;      //!< A counter, that gives all classes a Unique ClassID. Access to this Variable is to be Thread-Safe.
  static std::set<NewClassListBase>    _classes;        //!< A Set of all the classes in existance.
};

template<class T> class NewClassList : public NewClassListBase
{
public:
  NewClassList(const std::string& name);
  ~NewClassList();

private:
  //! the copy constructor will be hidden.
  NewClassList(const NewClassList& definer) {};
};

template <class T>
NewClassList<T>::NewClassList(const std::string& name)
    : NewClassListBase(name)
{}

template <class T>
NewClassList<T>::~NewClassList()
{}


#endif /* _NEW_CLASS_LIST_H */
